@@ -26,9 +26,9 @@ import math
 
 import numpy as np
 import pandas as pd
-from matplotlib import pyplot as plt
-
 from ambiance import Atmosphere
+from matplotlib import pyplot as plt
+from scipy import stats
 
 from cpacspy.cpacsfunctions import (
     add_float_vector,
@@ -39,12 +39,14 @@ from cpacspy.cpacsfunctions import (
 from cpacspy.utils import (
     AEROPERFORMANCE_XPATH,
     COEFS,
+    DAMPING_COEFS,
     PARAMS,
     PARAMS_COEFS,
-    DAMPING_COEFS,
     listify,
 )
 
+from cpacspy.utils import MSG_STAB_NEUTRAL, MSG_STAB_NOT_ENOUGH, MSG_STAB_ONE_PARAM
+
 
 def get_filter(df, alt_list, mach_list, aos_list, aoa_list):
     """Get a dataframe filter for a set of parameters lists."""
@@ -85,7 +87,7 @@ class AeroMap:
         self.name = uid
         self.description = ""
         self.atmospheric_model = "ISA"
-        self.df = pd.DataFrame(columns=PARAMS_COEFS)
+        self.df = pd.DataFrame(columns=PARAMS_COEFS, dtype=float)
 
         if create_new:
             self.name = uid
@@ -145,7 +147,7 @@ class AeroMap:
         self.df = pd.concat([self.df, df_param], axis=0)
 
     def get(self, list_of, alt=None, mach=None, aos=None, aoa=None):
-        """Get parameter or coefficient as a numpy vector with other parameters as filter (optional).
+        """Get parameter or coeffs as a numpy vector with other parameters as filter (optional).
 
         Args:
             list_of (str): Parameter or coefficient to get.
@@ -192,7 +194,7 @@ class AeroMap:
                 Must be one of the following: {"dp","dq","dr"}'
             )
 
-        if rates in ["posivitive", "pos", "p"]:
+        if rates in ["positive", "pos", "p"]:
             rates_name = "positiveRates"
         elif rates in ["negative", "neg", "n"]:
             rates_name = "negativeRates"
@@ -200,7 +202,7 @@ class AeroMap:
             raise ValueError(
                 f'Invalid rates "{rates}"! \n \
                              Must be written as: \n \
-                                 ("posivitive","pos","p" or "+") for positiveRates or \n \
+                                 ("positive","pos","p" or "+") for positiveRates or \n \
                                      ("negative","neg","n" or "-") for negativeRates'
             )
 
@@ -263,7 +265,7 @@ class AeroMap:
             "cms": cms,
         }
 
-        # Check if all colomn already exist
+        # Check if all column already exist
         for col in new_row:
             if col not in self.df.columns:
                 self.df[col] = np.nan
@@ -307,7 +309,7 @@ class AeroMap:
         cml=np.nan,
         cms=np.nan,
     ):
-        """Add coefficients to existing set of parmeter.
+        """Add coefficients to existing set of parameter.
 
         Args:
             alt (float): Altitude
@@ -335,7 +337,7 @@ class AeroMap:
         self.df.loc[filt, ["cd", "cl", "cs", "cmd", "cml", "cms"]] = [cd, cl, cs, cmd, cml, cms]
 
     def add_damping_derivatives(self, alt, mach, aos, aoa, coef, axis, value, rate=-1.0):
-        """Add a damping derivative coeficient for an existing set of alt,mach,aos,aoa.
+        """Add a damping derivative coefficients for an existing set of alt,mach,aos,aoa.
 
         Args:
             alt (float): Altitude
@@ -344,7 +346,7 @@ class AeroMap:
             aoa (float): Angle of attack
             coef (str) : equivalent aero coefficient (cd,cl,cs,cmd,cml,cms)
             axis (str) : axis of rotation (dp,dq,dr)
-            value (float) : value of the coeficient
+            value (float) : value of the coefficient
             rate (float, optional): Rotation rate. Defaults to -1.0.
 
         """
@@ -352,7 +354,7 @@ class AeroMap:
         damping_coef = "d" + coef + axis + "Star"
 
         if damping_coef not in DAMPING_COEFS:
-            raise ValueError(f'Damping coeficient "{damping_coef}" is not valid!')
+            raise ValueError(f'Damping coefficient "{damping_coef}" is not valid!')
 
         if rate < 0:
             col_name = "dampingDerivatives_negativeRates_" + damping_coef
@@ -386,7 +388,7 @@ class AeroMap:
         aoa_list = listify(aoa)
 
         filt = get_filter(self.df, alt_list, mach_list, aos_list, aoa_list)
-        self.df.loc[filt].plot(x=x_param, y=y_param, marker="o")
+        self.df.loc[filt].plot(x=x_param, y=y_param, ylabel=y_param, legend=False, marker="o")
         plt.show()
 
     def save(self):
@@ -414,12 +416,12 @@ class AeroMap:
                     add_float_vector(self.tixi, param_xpath, self.df[param].tolist())
                 else:
                     raise ValueError(
-                        "All the 4 parametres (alt,mach,aos,aoa) must not contains NaN value to \
+                        "All the 4 parameters (alt,mach,aos,aoa) must not contains NaN value to \
                             be saved in an aeroMap!"
                     )
             else:
                 raise ValueError(
-                    "All the 4 parametres (alt,mach,aos,aoa) must be define to save an aeroMap!"
+                    "All the 4 parameters (alt,mach,aos,aoa) must be define to save an aeroMap!"
                 )
 
         # Create and fill coefficients fields
@@ -431,8 +433,8 @@ class AeroMap:
                     add_float_vector(self.tixi, coef_xpath, self.df[coef].tolist())
                 else:
                     print(
-                        f'Warning: {coef} coeffiecient from "{self.uid}" aeroMap will not be written in the \
-                          CPACS file becuase it contains only NaN!'
+                        f'Warning: {coef} coefficient from "{self.uid}" aeroMap will not be \
+                          written in the CPACS file because it contains only NaN!'
                     )
 
         # Create and fill damping derivatives fields
@@ -446,8 +448,8 @@ class AeroMap:
                         add_float_vector(self.tixi, coef_xpath, self.df[col_name].tolist())
                     else:
                         print(
-                            f'Warning: {damping_coef} coeffiecient from "{self.uid}" aeroMap will \
-                              not be written in the CPACS file becuase it contains onlz NaN!'
+                            f'Warning: {damping_coef} coefficient from "{self.uid}" aeroMap will \
+                              not be written in the CPACS file because it contains onlz NaN!'
                         )
 
         # Create and fill incrementMap fields
@@ -522,7 +524,7 @@ class AeroMap:
         return cd0, e
 
     def calculate_forces(self, aircraft):
-        """Calculate forces and momement from coefficients"""
+        """Calculate forces and moment from coefficients"""
 
         COEF2FORCE_DICT = {"cd": "drag", "cl": "lift", "cs": "side"}
         COEF2MOMENT_DICT = {"cmd": "md", "cml": "ml", "cms": "ms"}
@@ -542,8 +544,8 @@ class AeroMap:
                 )
             else:
                 print(
-                    f"Warning: {COEF2FORCE_DICT[coef]} will not be calculated because there is no {coef} \
-                    coefficient in the aeroMap!"
+                    f"Warning: {COEF2FORCE_DICT[coef]} will not be calculated because there is \
+                    no {coef} coefficient in the aeroMap!"
                 )
 
         for coef in COEF2MOMENT_DICT:
@@ -553,10 +555,136 @@ class AeroMap:
                 )
             else:
                 print(
-                    f"Warning: {COEF2MOMENT_DICT[coef]} will not be calculated because there is no \
-                    {coef} coefficient in the aeroMap!"
+                    f"Warning: {COEF2MOMENT_DICT[coef]} will not be calculated because there is \
+                    no {coef} coefficient in the aeroMap!"
                 )
 
+    def check_longitudinal_stability(self, alt=None, mach=None, aos=None):
+        """Check longitudinal stability (cms vs aoa) with other parameters as filter (optional).
+        The stability is checked by making a linear regression on cms vs aoa and analyzing the
+        slope sign.
+
+        Args:
+            alt (list, optional): List of altitudes to filter. Defaults to None.
+            mach (list, optional): List of Mach numbers to filter. Defaults to None.
+            aos (list, optional): List of angle of sideslip to filter. Defaults to None.
+
+        Return:
+            stable (bool): Return if this set of conditions is stable (return False if neutral)
+            msg (str): Message description
+
+        """
+
+        msg = ""
+
+        alt_list = listify(alt)
+        mach_list = listify(mach)
+        aos_list = listify(aos)
+
+        if len(alt_list) > 1 or len(mach_list) > 1 or len(aos_list) > 1:
+            msg = MSG_STAB_ONE_PARAM
+
+        filt = get_filter(self.df, alt_list, mach_list, aos_list, [])
+        df_filt = self.df.loc[filt]
+        x = df_filt["angleOfAttack"].to_numpy()
+        y = df_filt["cms"].to_numpy()
+
+        if len(set(x)) < 2:
+            return None, MSG_STAB_NOT_ENOUGH
+
+        res = stats.linregress(x, y)
+
+        if res.slope == 0:
+            return False, MSG_STAB_NEUTRAL
+        elif res.slope < 0:
+            return True, msg
+        else:
+            return False, msg
+
+    def check_directional_stability(self, alt=None, mach=None, aoa=None):
+        """Check directional stability (cml vs aos) with other parameters as filter (optional).
+        The stability is checked by making a linear regression on cml vs aos and analyzing the
+        slope sign.
+
+        Args:
+            alt (list, optional): List of altitudes to filter. Defaults to None.
+            mach (list, optional): List of Mach numbers to filter. Defaults to None.
+            aoa (list, optional): List of angle of attack to filter. Defaults to None.
+
+        Return:
+            stable (bool): Return if this set of conditions is stable (return False if neutral)
+            msg (str): Message description
+
+        """
+
+        msg = ""
+
+        alt_list = listify(alt)
+        mach_list = listify(mach)
+        aoa_list = listify(aoa)
+
+        if len(alt_list) > 1 or len(mach_list) > 1 or len(aoa_list) > 1:
+            msg = MSG_STAB_ONE_PARAM
+
+        filt = get_filter(self.df, alt_list, mach_list, aoa_list, [])
+        df_filt = self.df.loc[filt]
+        x = df_filt["angleOfSideslip"].to_numpy()
+        y = df_filt["cml"].to_numpy()
+
+        if len(set(x)) < 2:
+            return None, MSG_STAB_NOT_ENOUGH
+
+        res = stats.linregress(x, y)
+
+        if res.slope == 0:
+            return False, MSG_STAB_NEUTRAL
+        elif res.slope < 0:
+            return True, msg
+        else:
+            return False, msg
+
+    def check_lateral_stability(self, alt=None, mach=None, aoa=None):
+        """Check lateral stability (cmd vs aos) with other parameters as filter (optional).
+        The stability is checked by making a linear regression on cmd vs aos and analyzing the
+        slope sign.
+
+        Args:
+            alt (list, optional): List of altitudes to filter. Defaults to None.
+            mach (list, optional): List of Mach numbers to filter. Defaults to None.
+            aoa (list, optional): List of angle of attack to filter. Defaults to None.
+
+        Return:
+            stable (bool): Return if this set of conditions is stable (return False if neutral)
+            msg (str): Message description
+
+        """
+
+        msg = ""
+
+        alt_list = listify(alt)
+        mach_list = listify(mach)
+        aoa_list = listify(aoa)
+
+        if len(alt_list) > 1 or len(mach_list) > 1 or len(aoa_list) > 1:
+            msg = MSG_STAB_ONE_PARAM
+
+        filt = get_filter(self.df, alt_list, mach_list, aoa_list, [])
+        df_filt = self.df.loc[filt]
+        x = df_filt["angleOfSideslip"].to_numpy()
+        y = df_filt["cmd"].to_numpy()
+
+        if len(set(x)) < 2:
+            return None, MSG_STAB_NOT_ENOUGH
+
+        res = stats.linregress(x, y)
+
+        if res.slope == 0:
+            return False, MSG_STAB_NEUTRAL
+        elif res.slope < 0:
+            return True, msg
+        else:
+            return False, msg
+
     def __str__(self):
 
         text_line = []
